@@ -1,5 +1,6 @@
 import atexit
 import logging
+from logging.config import dictConfig
 import os
 import sys
 import time
@@ -192,4 +193,21 @@ def bitcoind(debug,mining, docker_tag):
 
 
 if __name__ == "__main__":
+    # central and early configuring of logging
+    # see https://flask.palletsprojects.com/en/1.1.x/logging/#basic-configuration
+    dictConfig({
+        'version': 1,
+        'formatters': {'default': {
+            'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s',
+        }},
+        'handlers': {'wsgi': {
+            'class': 'logging.StreamHandler',
+            'stream': 'ext://flask.logging.wsgi_errors_stream',
+            'formatter': 'default'
+        }},
+        'root': {
+            'level': 'INFO',
+            'handlers': ['wsgi']
+        }
+    })
     cli()
\ No newline at end of file
