@@ -3,19 +3,20 @@ from rpc import *
 import requests
 
 if __name__ == '__main__':
-    cli_arr = detect_cli()
-    available_cli_arr = []
-    if len(cli_arr) > 0:
-        print("trying %d different configs" % len(cli_arr))
-        for cli in cli_arr:
+    conf_arr = detect_cli_confs()
+    available_conf_arr = []
+    if len(conf_arr) > 0:
+        print("trying %d different configs" % len(conf_arr))
+        for conf in conf_arr:
+            cli = BitcoinCLI(**conf)
             try:
                 print(cli.getmininginfo(timeout=1))
                 print("Yey! Bitcoin-cli found!")
-                available_cli_arr.append(cli)
+                available_conf_arr.append(cli)
             except requests.exceptions.RequestException:
                 print("can't connect")
             except Exception as e:
                 print("fail...", e)
     else:
         print("Bitcoin-cli not found :(")
-    print("\nDetected %d bitcoin daemons\n" % len(available_cli_arr))
+    print("\nDetected %d bitcoin daemons\n" % len(available_conf_arr))
