@@ -5,7 +5,6 @@ from pathlib import Path
 
 from dotenv import load_dotenv
 from flask import Flask
-from flask_qrcode import QRcode
 from flask_login import LoginManager, login_user
 
 from .descriptor import AddChecksum
@@ -38,7 +37,6 @@ def create_app(config="cryptoadvance.specter.config.DevelopmentConfig"):
 def init_app(app, specter=None):
     '''  see blogpost 19nd Feb 2020 '''
     app.logger.info("Initializing QRcode")
-    QRcode(app) # enable qr codes generation
     # Login via Flask-Login
     app.logger.info("Initializing LoginManager")
     login_manager = LoginManager()
@@ -69,7 +67,15 @@ def init_app(app, specter=None):
         from . import controller
     return app
 
-
+def create_and_init():
+    ''' This method can be used to fill the FLASK_APP-env variable like
+        export FLASK_APP="src/cryptoadvance/specter/server:create_and_init()"
+        See Development.md to use this for debugging
+    '''
+    app = create_app()
+    app.app_context().push()
+    init_app(app)
+    return app
 
 
 DATA_FOLDER = "~/.specter"
@@ -101,4 +107,4 @@ class AuthenticatedUser:
         return False
 
     def get_id(self):
-        return "there-is-only-one-User"
\ No newline at end of file
+        return "there-is-only-one-User"
